import traceback

from dask.distributed import Client

from plantit_cli.runner.runner import Runner
from plantit_cli.config import Config
from plantit_cli.utils import update_status, run_container_for_directory, run_containers_for_files, \
    run_container


class LocalRunner(Runner):
    name = "local"

    def __init__(self, token: str):
        self.token = token

    def run(self, plan: Config):
        update_status(plan, 3, f"Starting run '{plan.identifier}' with '{self.name}' executor")
        try:
            with Client() as client:
                if plan.clone is not None and plan.clone is not '':
                    self.clone_repo(plan)

                if plan.input:
                    print(f"Pulling inputs for run '{plan.identifier}'")
                    input_directory = self.pull_input(plan)
                    if plan.input['kind'].lower() == 'directory':
                        run_container_for_directory(plan, client, input_directory)
                    elif plan.input['kind'].lower() == 'file':
                        run_containers_for_files(plan, client, input_directory)
                    else:
                        raise ValueError(f"Value of 'input.kind' must be either 'file' or 'directory'")
                else:
                    print(f"Executing flow for run '{plan.identifier}'")
                    run_container(plan, client)

                if plan.output:
                    print(f"Pushing outputs for run '{plan.identifier}'")
                    self.push_output(plan)
        except Exception:
            update_status(plan, 3, f"Run '{plan.identifier}' failed: {traceback.format_exc()}")
            return
